<?php
namespace Teknisa\EST\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Est31100PosicaoDeEstoqueDia {
    protected $reportService;
    protected $dataSourceManager;
    protected $nrorg;

    public function __construct(ReportService $reportService, $dataSourceManager, $environment) {
        $this->reportService       = $reportService;
        $this->dataSourceManager   = $dataSourceManager;
        $this->nrorg = $environment->getNrorgTrab();
    }

    public function generateReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $reportName = $row['NMRELATORIO'];
        $reportStrategy = $this->getReportStrategyReport($reportName);
        $parametersRow = array();
        
        foreach ($reportStrategy['parameterMapping'] as $fieldName => $paramName) {
            if ($row->has($fieldName)) {
                if($reportName === 'EST31119' || $reportName === 'EST32800' || $reportName === 'EST32804' || $reportName === 'EST32808' )  {
                    if($fieldName === 'NVTOT') {
                        $parametersRow[$paramName] = $this->getNvTot($row->get($fieldName));
                    }
                    else if($fieldName === 'ORDENANOME')  $parametersRow[$paramName]  = ($row->get($fieldName) == 'S') ? '0' : '10';
                    else if($fieldName === 'CDALMOXARIFE') $parametersRow[$paramName] = ($row->get($fieldName) == ' ') ? '' : $row->get($fieldName);
                    else if($fieldName === 'CDLOCALESTOQ') $parametersRow[$paramName] = ($row->get($fieldName) == ' ') ? '' : $row->get($fieldName);
                    else if($fieldName === 'DATAPOS')     {
                        $data = str_replace('/', '-', $row->get($fieldName));
                        if($reportName == 'EST31119') {
                            $data = date('m/d/Y', strtotime($data));
                        }
                        else {
                            $data = date('d/m/Y', strtotime($data));
                        }
                        $parametersRow[$paramName] = $data;
                        if($reportName === 'EST32804'  || $reportName === 'EST32808' ) {
                            $parametersRow['P_DTCUSTPROD'] = $data; 
                            $parametersRow['P_CDGRUPFILI'] = $row->get('CDGRUPFILI');
                        }
                    }
                    else $parametersRow[$paramName] = $row->get($fieldName);
                }
                else $parametersRow[$paramName] = $row->get($fieldName);
            }
        }
        $parametersRow['P_NRORG']  = $this->nrorg;
        
        $parametersRow['__format'] = $row["__FORMAT"];
        
        $birtURL = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow);
        
        $response->addMessage(new DTO\Response\Message($birtURL));
    }

    private function getNvTot($nivelEscolhido) {
        $nivelEscolhido  = $nivelEscolhido - 1;
        $filterCriteria  = new FilterCriteria('paramgeral',array());
        $dataSet         = $this->dataSourceManager->findBy($filterCriteria);
        $mascaraProdutos = $dataSet->getRows()[0]['CDPICTPROD'];
        $arrayMascara    = explode(".", $mascaraProdutos);
        for($count = 0, $nvTot = 0; $count <= $nivelEscolhido; $count++) {
            $nvTot += strlen($arrayMascara[$count]);
        }
        return $nvTot;
    }

    private function getReportStrategyReport($reportName) {
        $reportStrategy = array (
            'strategy' => 'BIRT'
        );
        switch($reportName) {
            case 'EST31100' || 'EST31100_noHeader':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "P_CDFILIAL",
                    "CDALMOXARIFE" => "P_CDALMOXARIFE",
                    "CDLOCALESTOQ" => "P_CDLOCALESTOQ",
                    "DATAPOS" => "P_DTPOSIESTQ",
                    "CDTIPOCUSTO" => "P_CDTIPOCUSTO",
                    "CDARVPRODINI" => "P_CDARVPRODINI",
                    "CDARVPRODFIN" => "P_CDARVPRODFIN",
                    "NVTOT" => "P_NVTOT",
                    "ORDENANOME" => "P_ORDENANOME",
                    "NRORG" => "P_NRORG",
                    "NRTPARVPROD" => "P_NRTPARVPROD",
                    "NRARVOREPROD" => "P_NRARVOREPROD"
                );
                break;
            case 'EST31119':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "P_CDFILIAL",
                    "CDALMOXARIFE" => "P_CDALMOXARIFE",
                    "CDLOCALESTOQ" => "P_CDLOCALESTOQ",
                    "DATAPOS" => "P_DTPOSIESTQ",
                    "CDTIPOCUSTO" => "P_CDTIPOCUSTO",
                    "CDARVPRODINI" => "P_CDARVPRODINI",
                    "CDARVPRODFIN" => "P_CDARVPRODFIN",
                    "NVTOT" => "P_NVTOT",
                    "ORDENANOME" => "P_ORDENANOME",
                    "DSTIPOCUSTO" => "P_DSTIPOCUSTO",
                    "NRORG" => "P_NRORG"
                );
                break;
            case 'EST32800':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "P_CDFILIAL",
                    "DATAPOS" => "P_DTPOSIESTQ",
                    "CDTIPOCUSTO" => "P_CDTIPOCUSTO",
                    "CDARVPRODINI" => "P_CDARVPRODINI",
                    "CDARVPRODFIN" => "P_CDARVPRODFIN",
                    "NVTOT" => "P_NVTOT",
                    "ORDENANOME" => "P_ORDENANOME",
                    "DSTIPOCUSTO" => "P_DSTIPOCUSTO",
                    "NRORG" => "P_NRORG"
                );
                break;
            case 'EST32804':
                $reportStrategy['parameterMapping'] = array (
                    "CDGRUPFILI" => "P_CDGRUPFILI",
                    "DATAPOS" => "P_DTPOSIESTQ",
                    "CDTIPOCUSTO" => "P_CDTIPOCUSTO",
                    "CDARVPRODINI" => "P_CDARVPRODINI",
                    "CDARVPRODFIN" => "P_CDARVPRODFIN",
                    "NVTOT" => "P_NVTOT",
                    "ORDENANOME" => "P_ORDENANOME",
                    "DSTIPOCUSTO" => "P_DSTIPOCUSTO",
                    "NRORG" => "P_NRORG"
                );
                break;
            case 'EST32808':
                 $reportStrategy['parameterMapping'] = array (
                    "CDGRUPFILI" => "P_CDGRUPFILI",
                    "DATAPOS" => "P_DTPOSIESTQ",
                    "CDTIPOCUSTO" => "P_CDTIPOCUSTO",
                    "CDARVPRODINI" => "P_CDARVPRODINI",
                    "CDARVPRODFIN" => "P_CDARVPRODFIN",
                    "NVTOT" => "P_NVTOT",
                    "ORDENANOME" => "P_ORDENANOME",
                    "DSTIPOCUSTO" => "P_DSTIPOCUSTO",
                    "NRORG" => "P_NRORG"
                );
                
                break;
        }
        return $reportStrategy;
    }
}